class IncrementSelector {
    static int minstd_next(int v) {
        return (int)((16807L * v) % 2147483647L);
    }

    static boolean isprime(int v) {
        if (v == 1)
            return false;
        if (v == 2)
            return true;
        if ((v % 2) == 0)
            return false;

        int j;
        for(j = 3; j * j <= v; j += 2) {
            if((v % j) == 0)
                return false;
        }

        return true;

    }

    static int nextprime(int m) {
        while(!isprime(m))
            ++m;
        return m;
    }

    /** Selecting an increment.
     *
     * The set of increments is selected in a manner so as to avoid
     * sequences which are dependent from one another.
     *
     * The algorithm to generate this table is as follows:
     *
     * (1)  Generate a sequence of pseudo-random integers using the
     * MINSTD (Park-Miller) random variable generator with starting seed
     * 1.
     *
     * (2)  For each such value v, find w := nextprime(v), where
     * nextprime(v) is the smallest prime larger than or equal to w.
     *
     * (3)  If w is in the range [2^24, 2^31 - 1), append it to the list
     * of increments, otherwise skip it.
     *
     * Repeat until 10000 increments are found.
     *
     * There are no duplicate entries among the first 10000 increments.
     */
    int GetIncrement(int trial_number) {
        int v = 1;
        int c = 0;
        do {
            do {
                v = minstd_next(v);
            } while (v < (1 << 24) || v >= ((1 << 31) - 18));
            if (c == trial_number)
                return nextprime(v);
            ++c;
        } while(true);
    }
}
