    /* package com.expway.fectest; */

import static java.lang.System.out;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

public class LossVectorGenerator {


    static double probaP;
    static double probaQ;
    static double goodBLER;
    static double badBLER;
    static int subSamplingFrequency = 1;
    static int trial_number;
    static long m = 1000000;
    static long offset;
    static long n;
    private static String vectorFileName;

    /**
     * prng as defined in rfc6330#section 5.5.
     */
    Random prng;

    /**
     * a mapping of case number to increment
     */
    IncrementSelector isel;

    /**
     * true if we are in the good markov state, false for the bad state.
     */
    private boolean inGoodState;

    /**
     * @param args
     * @throws IOException
     */
    public static void main(final String[] args) throws IOException {

        assert args.length == 8;

        probaP = Double.parseDouble(args[0]);
        probaQ = Double.parseDouble(args[1]);
        goodBLER = Double.parseDouble(args[2]);
        badBLER = Double.parseDouble(args[3]);
        subSamplingFrequency = Integer.parseInt(args[4]);
        n = Long.parseLong(args[5]);
        trial_number = Integer.parseInt(args[6]);
        vectorFileName = args[7];


        assert probaP>=0:"probaP<0";
        assert probaQ>=0:"probaQ<0";
        assert probaP<=1:"probaP>1";
        assert probaQ<=1:"probaQ>1";
        assert goodBLER>=0:"goodBLER<0";
        assert badBLER>=0:"badBLER<0";
        assert goodBLER<=1:"goodBLER>1";
        assert badBLER<=1:"badBLER>1";
        assert subSamplingFrequency==1 ||subSamplingFrequency==4 : "subSamplingFrequency: only 1 and 4 authorized";
        assert n>0: "n<=0";
        assert offset>=0: "offset<0";


        new LossVectorGenerator().launchTest();
    }

    private void launchTest() throws IOException {


        System.out.println("Launching loss vector generation with n=" + n
                + " probaP=" + probaP
                + " probaQ=" + probaQ
                + " goodBLER=" + goodBLER
                + " badBLER=" + badBLER
                + " trial_number=" + trial_number
                + " subSamplingFrequency=" + subSamplingFrequency);


        double pGoodState = probaQ/(probaP + probaQ);
        double bler = goodBLER *  pGoodState  + badBLER * (1- pGoodState);

        out.println("P good state =" + pGoodState);
        out.println("P bad state =" + (1-pGoodState));
        out.println("Bad state average length =" + (1/probaQ));
        out.println("Theoritical BLER =" + bler);


        long totalCount = 0;
        long totalLost = 0;
        long totalReceived = 0;

        isel = new IncrementSelector();

        prng = new Random(isel.GetIncrement(trial_number), trial_number, m);

        Writer writer = new BufferedWriter(
                new FileWriter(vectorFileName));

        writer.write(n + "\n");

        long totalSent = 0;
        
        long numPacketToSend = n * subSamplingFrequency;
        
        inGoodState = (prng.rng() < pGoodState * m);
        while (totalSent < numPacketToSend) {
            move();
            boolean isLost = isLost();
            totalSent++;

            if((totalSent-1) % subSamplingFrequency == 0) {
                totalCount ++;

                if (isLost) {
                    writer.write("1");
                    totalLost ++;

                } else {
                    writer.write("0");
                    totalReceived ++;
                }
            }
        }

        writer.close();
        assert totalReceived + totalLost == n : "totalReceived + totalLost != n ";
        double measuredBler = (double) totalLost/n;
        out.println("Loss: " + totalLost + "/" + n);
        out.println("Measured BLER: " + measuredBler);


    }

    /**
     * Iterates on the markov chain.
     */
    private void move() {
        if (inGoodState) {
            //from the good state
            //testing the p transition
            if (prng.rng() < probaP * m) {
                inGoodState = false;
            }
        } else {
            //from the bad state
            //testing the q transition
            if(prng.rng() < probaQ * m) {
                inGoodState = true;
            }
        }
    }

    /**
     * Tests if the next packet is lost.
     * @returns true if the packet is lost.
     */
    private boolean isLost() {

        double currentBLER = inGoodState ? goodBLER : badBLER;
        return prng.rng() < currentBLER * m;


    }

}
