#!/bin/sh
CP=cp
MT=LossVectorGenerator
err_trace_prefix="error_trace"


# Markov parameters for 3kph
lsp[1]=0.0058; lsq[1]=0.3613; lsg[1]=0.0003; lsb[1]=0.5947; lsloss[1]="01"; lsspeed[1]="3kph"
lsp[2]=0.0180; lsq[2]=0.2401; lsg[2]=0.0006; lsb[2]=0.7054; lsloss[2]="05"; lsspeed[2]="3kph"
lsp[3]=0.0279; lsq[3]=0.2090; lsg[3]=0.0056; lsb[3]=0.8230; lsloss[3]="10"; lsspeed[3]="3kph"
lsp[4]=0.0461; lsq[4]=0.1680; lsg[4]=0.0116; lsb[4]=0.8920; lsloss[4]="20"; lsspeed[4]="3kph"
# Markov parameters for 120kph
lsp[5]=0.0606; lsq[5]=0.9430; lsg[5]=0.0000; lsb[5]=0.1731; lsloss[5]="01"; lsspeed[5]="120kph"
lsp[6]=0.2707; lsq[6]=0.7095; lsg[6]=0.0000; lsb[6]=0.1954; lsloss[6]="05"; lsspeed[6]="120kph"
lsp[7]=0.4648; lsq[7]=0.5095; lsg[7]=0.0000; lsb[7]=0.2233; lsloss[7]="10"; lsspeed[7]="120kph"
lsp[8]=0.3560; lsq[8]=0.6329; lsg[8]=0.0972; lsb[8]=0.4040; lsloss[8]="20"; lsspeed[8]="120kph"

# Two bearers info (398Kpbs==1066Kbps) 
#      rate        ; subsampling; total packets for 24h
bearer[0]="266kps";  bearss[0]=4; bearpkts[0]=2160000
bearer[1]="1066kps"; bearss[1]=1; bearpkts[1]=8640000

# Values of N' for segment size of 1, 2 and 4 secs.
# Divide by subsampling factor to get N 
segN[1]=100; segN[2]=200; segN[3]=400

for i in `seq 16` 
do
    mci=$(( (($i-1) / 2) + 1  )) # Markov chain index
    beari=$(( ($i-1) % 2 ))      # Bearer index
    trace=${err_trace_prefix}_${lsloss[$mci]}_${lsspeed[$mci]}_${bearer[$beari]}
    bler=`java ${MT} ${lsp[${mci}]} ${lsq[$mci]} ${lsg[$mci]} ${lsb[$mci]} ${bearss[$beari]} ${bearpkts[$beari]} 0 ${trace}.txt > ${trace}-out.txt`
    tail -n 1 ${trace}-out.txt | awk '{ print $3 }'
    md5=`openssl md5 ${trace}.txt`
    echo $md5 $bler > ${trace}.md5
    cat ${trace}.md5
    for j in `seq 3` 
    do
        seglen=$(( 2 ** ($j-1) )) # segment length in seconds
        currentN=$(( ${segN[$j]} / ${bearss[$beari]} )) # Compute N
        python streamIdealK.py ${trace}.txt ${currentN} ${seglen} ${trace}_seg${seglen}sec > ${trace}_seg${seglen}sec-results.txt
        tail -n 1 ${trace}_seg${seglen}sec-results.txt
    done
done
